package org.processmining.activityclusterarraycreator.algorithms;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarray.models.impl.ActivityClusterArrayFactory;
import org.processmining.activityclusterarraycreator.parameters.DiscoverClustersFromEventLogParameters;
import org.processmining.framework.plugin.PluginContext;

public class DiscoverClustersFromEventLogAlgorithm {

	public ActivityClusterArray apply(PluginContext context, XLog log,
			DiscoverClustersFromEventLogParameters parameters) {
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log, parameters.getClassifier());
		List<XEventClass> activities = new ArrayList<XEventClass>(logInfo.getEventClasses().getClasses());
		Set<Set<XEventClass>> clusters = new HashSet<Set<XEventClass>>();
		XConceptExtension conceptExtension = XConceptExtension.instance();
		String label = conceptExtension.extractName(log);
		if (label == null || label.isEmpty()) {
			label = "<unnamed log>";
		}
		Map<XEventClass, Set<XEventClass>> clusterMap = new HashMap<XEventClass, Set<XEventClass>>();
		for (int i = 0; i < activities.size(); i++) {
			XEventClass activity = activities.get(i);
			Set<XEventClass> cluster = new HashSet<XEventClass>();
			cluster.add(activity);
			clusterMap.put(activity, cluster);
		}
		for (XTrace trace : log) {
			for (int j = 0; j < trace.size(); j++) {
				Set<XEventClass> cluster = clusterMap.get(logInfo.getEventClasses().getClassOf(trace.get(j)));
				if (j > 1) {
					cluster.add(logInfo.getEventClasses().getClassOf(trace.get(j - 1)));
				}
				if (j + 1 < trace.size()) {
					cluster.add(logInfo.getEventClasses().getClassOf(trace.get(j + 1)));
				}
			}
		}
		clusters.addAll(clusterMap.values());
		ActivityClusterArray clusterArray = ActivityClusterArrayFactory.createActivityClusterArray();
		clusterArray.init(label, new HashSet<XEventClass>(activities));
		for (Set<XEventClass> cluster : clusters) {
			clusterArray.addCluster(cluster);
		}
		return clusterArray;
	}
}
