package org.processmining.activityclusterarraycreator.dialogs;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Dimension;
import java.util.HashSet;
import java.util.Set;

import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarray.models.impl.ActivityClusterArrayFactory;
import org.processmining.activityclusterarray.plugins.VisualizeActivityClusterArrayPlugin;
import org.processmining.activityclusterarraycreator.algorithms.ConvertCausalActivityGraphToActivityClusterArrayAlgorithm;
import org.processmining.activityclusterarraycreator.parameters.ConvertCausalActivityGraphToActivityClusterArrayParameters;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.plugin.PluginContext;

import com.fluxicon.slickerbox.components.NiceSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class ConvertCausalActivityGraphToActivityClusterArrayDialog extends JPanel {

	/**
	         *
	         */
	private static final long serialVersionUID = -5552669985865681115L;

	public ConvertCausalActivityGraphToActivityClusterArrayDialog(final UIPluginContext context,
			final CausalActivityGraph graph, final ConvertCausalActivityGraphToActivityClusterArrayParameters parameters) {
		double size[][] = { { TableLayoutConstants.FILL }, { TableLayoutConstants.FILL, 30, 30 } };
		setLayout(new TableLayout(size));

		final ActivityClusterArray clusterArray = ActivityClusterArrayFactory.createActivityClusterArray();
		clusterArray.init(graph.getLabel(), new HashSet<XEventClass>(graph.getActivities()));
		updateClusterArray(context, clusterArray, graph, parameters);

		final JPanel panel = new JPanel();
		panel.setPreferredSize(new Dimension(100, 100));
		double panelSize[][] = { { TableLayoutConstants.FILL }, { TableLayoutConstants.FILL } };
		panel.setLayout(new TableLayout(panelSize));
		add(panel, "0, 0");

		final VisualizeActivityClusterArrayPlugin visualizer = new VisualizeActivityClusterArrayPlugin();
		panel.add(visualizer.runUI(context, clusterArray), "0, 0");

		final NiceSlider thresholdSlider = SlickerFactory.instance().createNiceIntegerSlider("Target cluster size ", 2,
				parameters.getMaxThreshold(), parameters.getClusterThreshold(), Orientation.HORIZONTAL);
		thresholdSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setClusterThreshold(thresholdSlider.getSlider().getValue());
				panel.removeAll();
				updateClusterArray(context, clusterArray, graph, parameters);
				panel.add(visualizer.runUI(context, clusterArray), "0, 0");
				revalidate();
				repaint();
			}
		});
		add(thresholdSlider, "0, 1");

		final NiceSlider factorSilder = SlickerFactory.instance().createNiceDoubleSlider("Weight threshold", 0.0, 1.0,  parameters.getClusterFactor(),
				Orientation.HORIZONTAL);
		factorSilder.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setClusterFactor(factorSilder.getSlider().getValue() * 1.0 / 10000.0);
				panel.removeAll();
				updateClusterArray(context, clusterArray, graph, parameters);
				panel.add(visualizer.runUI(context, clusterArray), "0, 0");
				revalidate();
				repaint();
			}
		});
		add(factorSilder, "0, 2");
}

	private void updateClusterArray(PluginContext context, ActivityClusterArray clusterArray, CausalActivityGraph graph,
			ConvertCausalActivityGraphToActivityClusterArrayParameters parameters) {
		ConvertCausalActivityGraphToActivityClusterArrayAlgorithm converter = new ConvertCausalActivityGraphToActivityClusterArrayAlgorithm();
		ActivityClusterArray dummy = converter.apply(context, graph, parameters);
		while (!clusterArray.getClusters().isEmpty()) {
			clusterArray.removeCluster(0);
		}
		for (Set<XEventClass> cluster : dummy.getClusters()) {
			clusterArray.addCluster(cluster);
			clusterArray.setInputs(cluster, dummy.getInputs(cluster));
			clusterArray.setOutputs(cluster, dummy.getOutputs(cluster));
		}
	}

}
