package org.processmining.activityclusterarraycreator.metrics.impl;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.math3.stat.descriptive.SummaryStatistics;
import org.deckfour.xes.classification.XEventClass;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarraycreator.metrics.ActivityClusterArrayMetric;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.framework.plugin.PluginContext;

public class ActivityClusterArrayBalanceMetric implements ActivityClusterArrayMetric {

	private double weight;
	
	public ActivityClusterArrayBalanceMetric(double weight) {
		this.setWeight(weight);
	}
	
	public double compute(PluginContext context, ActivityClusterArray clusters, CausalActivityGraph graph) {
		
		if (weight == 0.0) {
			return 0.0;
		}
		
		if(clusters.getClusters().size() == 1)
			return 1.0;
		
		SummaryStatistics stats = new SummaryStatistics();
		
		Set<XEventClass> union = new HashSet<XEventClass>();
		for(Set<XEventClass> cluster: clusters.getClusters()){
			stats.addValue(cluster.size());
			union.addAll(cluster);
		}
		
		double balance = 1.0 - ( stats.getStandardDeviation() / ( union.size() / 2 ) );
		
		return balance;
	}

	public double getWeight() {
		return weight;
	}

	public void setWeight(double weight) {
		this.weight = weight;
	}

}
