package org.processmining.activityclusterarraycreator.metrics.impl;

import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarraycreator.metrics.ActivityClusterArrayMetric;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.framework.plugin.PluginContext;

public class ActivityClusterArrayCouplingMetric implements ActivityClusterArrayMetric {

	private double weight;

	public ActivityClusterArrayCouplingMetric(double weight) {
		this.setWeight(weight);
	}

	public double compute(PluginContext context, ActivityClusterArray clusters, CausalActivityGraph graph) {

		if(clusters.getClusters().size() == 1) {
			return 1.0;
		}
		
		double coupling = 0.0;
		
		for (Set<XEventClass> firstCluster : clusters.getClusters()) {
			for (Set<XEventClass> secondCluster : clusters.getClusters()) {
				coupling += compute(context, firstCluster, secondCluster, graph);
			}
		}
		return 1.0 - (coupling / (clusters.getClusters().size() * (clusters.getClusters().size() - 1)));
	}

	public double compute(PluginContext context, Set<XEventClass> firstCluster, Set<XEventClass> secondCluster, CausalActivityGraph graph) {
		double coupling = 0.0;
		if (firstCluster != secondCluster) {
			double clusterCoupling = 0.0;
			for (XEventClass firstActivity : firstCluster) {
				for (XEventClass secondActivity : secondCluster) {
					clusterCoupling += graph.getCausality(firstActivity, secondActivity);
					clusterCoupling += graph.getCausality(secondActivity, firstActivity);
				}
			}
			coupling += (clusterCoupling / (2 * firstCluster.size() * secondCluster.size()));
		}
		return coupling;
	}	
	
	public double getWeight() {
		return weight;
	}

	public void setWeight(double weight) {
		this.weight = weight;
	}

}
