package org.processmining.activityclusterarraycreator.metrics.impl;

import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarraycreator.metrics.ActivityClusterArrayMetric;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.framework.plugin.PluginContext;

public class ActivityClusterArrayOverlapMetric implements ActivityClusterArrayMetric{

	private double weight;

	public ActivityClusterArrayOverlapMetric(double weight) {
		this.setWeight(weight);
	}

	public double compute(PluginContext context, ActivityClusterArray clusters, CausalActivityGraph graph) {
		double minOverlap = 0.0;
		for (int i = 0; i < clusters.getClusters().size(); i++) {
			Set<XEventClass> firstCluster = clusters.getCluster(i);
			for (int j = i + 1; j < clusters.getClusters().size(); j++) {
				Set<XEventClass> secondCluster = clusters.getCluster(j);
				Set<XEventClass> isect = new HashSet<XEventClass>(firstCluster);
				isect.retainAll(secondCluster);
				int size = (firstCluster.size() < secondCluster.size()) ? firstCluster.size() : secondCluster.size();
				double overlap = 1.0 - ((1.0 * isect.size()) / size);
				if (overlap < minOverlap) {
					minOverlap = overlap;
				}
			}
		}
		return minOverlap;
	}

	public double getWeight() {
		// TODO Auto-generated method stub
		return weight;
	}

	public void setWeight(double weight) {
		this.weight = weight;
	}

}
