package org.processmining.activityclusterarraycreator.models.graph;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarraycreator.connections.ConvertCausalActivityGraphToActivityClusterArrayConnection;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.AttributeMap.ArrowType;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphElement;
import org.processmining.models.graphbased.directed.DirectedGraphNode;

public class ActivityClusterArrayGraph extends AbstractDirectedGraph<ActivityClusterArrayNode, ActivityClusterArrayEdge> {

	private ActivityClusterArray clusters;
	private CausalActivityGraph graph;
	private Map<Object, ActivityClusterArrayNode> nodeMap = null;
	private Map<Pair<Object, Object>, ActivityClusterArrayEdge> edgeMap = null;

	public ActivityClusterArrayGraph(PluginContext context, ActivityClusterArray clusters) {
		this.clusters = clusters;
		this.graph = null;
		if (context != null) {
			try {
				Collection<ConvertCausalActivityGraphToActivityClusterArrayConnection> connections = context
						.getConnectionManager().getConnections(
								ConvertCausalActivityGraphToActivityClusterArrayConnection.class, context, clusters);
				for (ConvertCausalActivityGraphToActivityClusterArrayConnection connection : connections) {
					if (connection.getObjectWithRole(
							ConvertCausalActivityGraphToActivityClusterArrayConnection.CLUSTERS).equals(clusters)) {
						this.graph = connection
								.getObjectWithRole(ConvertCausalActivityGraphToActivityClusterArrayConnection.GRAPH);
						continue;
					}
				}
			} catch (Exception e) {
				// Don't care.
			}
		}
	}

	public void init() {
		if (nodeMap == null && edgeMap == null) {
			nodeMap = new HashMap<Object, ActivityClusterArrayNode>();
			edgeMap = new HashMap<Pair<Object, Object>, ActivityClusterArrayEdge>();
			int size = clusters.getClusters().size();
			for (int i = 0; i < size; i++) {
				nodeMap.put(i, new ActivityClusterArrayNode(this, i));
				Set<XEventClass> cluster = clusters.getCluster(i);
				for (XEventClass activity : cluster) {
					boolean isIn = clusters.getOutputs(cluster).contains(activity);
					boolean isOut = clusters.getInputs(cluster).contains(activity);
					if (graph != null) {
						Set<XEventClass> pre = graph.getPreSetActivities(activity);
						pre.retainAll(cluster);
						if (!pre.isEmpty()) {
							isOut = true;
						}
						Set<XEventClass> post = graph.getPostSetActivities(activity);
						post.retainAll(cluster);
						if (!post.isEmpty()) {
							isIn = true;
						}
					}
					if (!nodeMap.containsKey(activity)) {
						nodeMap.put(activity, new ActivityClusterArrayNode(this, activity));
					}
					if (isIn) {
						edgeMap.put(new Pair<Object, Object>(i, activity), new ActivityClusterArrayEdge(nodeMap.get(i),
								nodeMap.get(activity), ArrowType.ARROWTYPE_CLASSIC));
					}
					if (isOut) {
						edgeMap.put(new Pair<Object, Object>(activity, i), new ActivityClusterArrayEdge(
								nodeMap.get(activity), nodeMap.get(i), ArrowType.ARROWTYPE_CLASSIC));
					}
					if (!isIn && !isOut) {
						edgeMap.put(new Pair<Object, Object>(i, activity), new ActivityClusterArrayEdge(nodeMap.get(i),
								nodeMap.get(activity), ArrowType.ARROWTYPE_NONE));
					}
				}
			}
		}
	}

	public Set<ActivityClusterArrayNode> getNodes() {
		init();
		return new HashSet<ActivityClusterArrayNode>(nodeMap.values());
	}

	public Set<ActivityClusterArrayEdge> getEdges() {
		init();
		return new HashSet<ActivityClusterArrayEdge>(edgeMap.values());
	}

	public void removeNode(DirectedGraphNode cell) {
	}

	protected AbstractDirectedGraph<ActivityClusterArrayNode, ActivityClusterArrayEdge> getEmptyClone() {
		return new ActivityClusterArrayGraph(null, clusters);
	}

	protected Map<DirectedGraphElement, DirectedGraphElement> cloneFrom(
			DirectedGraph<ActivityClusterArrayNode, ActivityClusterArrayEdge> graph) {
		ActivityClusterArrayGraph clone = new ActivityClusterArrayGraph(null, this.clusters);
		clone.init();
		Map<DirectedGraphElement, DirectedGraphElement> map = new HashMap<DirectedGraphElement, DirectedGraphElement>();
		for (Object activity : nodeMap.keySet()) {
			map.put(this.nodeMap.get(activity), clone.nodeMap.get(activity));
		}
		return map;
	}

	public void removeEdge(@SuppressWarnings("rawtypes") DirectedGraphEdge edge) {
	}

}
