package org.processmining.activityclusterarraycreator.modifiers;

import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarraycreator.metrics.ActivityClusterArrayMetric;
import org.processmining.activityclusterarraycreator.metrics.impl.ActivityClusterArrayBalanceMetric;
import org.processmining.activityclusterarraycreator.metrics.impl.ActivityClusterArrayCohesionMetric;
import org.processmining.activityclusterarraycreator.metrics.impl.ActivityClusterArrayCouplingMetric;
import org.processmining.activityclusterarraycreator.metrics.impl.ActivityClusterArrayOverlapMetric;
import org.processmining.activityclusterarraycreator.metrics.impl.ActivityClusterArrayWeightedMetric;
import org.processmining.basicutils.parameters.impl.PluginParametersImpl;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.framework.plugin.PluginContext;

public class ActivityClusterArrayModifierParameters extends PluginParametersImpl {

	private int nrOfClusters;
	private int cohesionWeight;
	private int couplingWeight;
	private int balanceWeight;
	private int overlapWeight;

	public ActivityClusterArrayModifierParameters(ActivityClusterArray clusters) {
		super();
		setCohesionWeight(100);
		setCouplingWeight(100);
		setBalanceWeight(100);
		setOverlapWeight(100);
		setNrOfClusters(clusters.getClusters().size() /2);		
	}
	
	public ActivityClusterArrayModifierParameters(ActivityClusterArrayModifierParameters parameters) {
		super(parameters);
		setCohesionWeight(parameters.getCohesionWeight());
		setCouplingWeight(parameters.getCouplingWeight());
		setBalanceWeight(parameters.getBalanceWeight());
		setOverlapWeight(parameters.getOverlapWeight());
		setNrOfClusters(parameters.getNrOfClusters());
	}
	
	public boolean equals(Object object) {
		if (object instanceof ActivityClusterArrayModifierParameters) {
			ActivityClusterArrayModifierParameters parameters = (ActivityClusterArrayModifierParameters) object;
			return super.equals(parameters) && getCohesionWeight() == parameters.getCohesionWeight() &&
					getCouplingWeight() == parameters.getCouplingWeight() &&
					getBalanceWeight() == parameters.getBalanceWeight() &&
					getOverlapWeight() == parameters.getOverlapWeight() &&
					getNrOfClusters() == parameters.getNrOfClusters();
		}
		return false;
	}
	
	public int getCohesionWeight() {
		return cohesionWeight;
	}

	public void setCohesionWeight(int cohesionWeight) {
		this.cohesionWeight = cohesionWeight;
	}

	public int getCouplingWeight() {
		return couplingWeight;
	}

	public void setCouplingWeight(int couplingWeight) {
		this.couplingWeight = couplingWeight;
	}

	public int getBalanceWeight() {
		return balanceWeight;
	}

	public void setBalanceWeight(int balanceWeight) {
		this.balanceWeight = balanceWeight;
	}

	public int getNrOfClusters() {
		return nrOfClusters;
	}

	public void setNrOfClusters(int nrOfClusters) {
		this.nrOfClusters = nrOfClusters;
	}

	public int getOverlapWeight() {
		return overlapWeight;
	}

	public void setOverlapWeight(int overlapWeight) {
		this.overlapWeight = overlapWeight;
	}

	public double getScore(PluginContext context, ActivityClusterArray clusters, CausalActivityGraph graph) {

		if ((cohesionWeight == 0 && couplingWeight == 0 && balanceWeight == 0 && overlapWeight == 0))
			return 0.0;

		ActivityClusterArrayMetric metric = new ActivityClusterArrayWeightedMetric(
				new ActivityClusterArrayCohesionMetric(cohesionWeight),
				new ActivityClusterArrayCouplingMetric(couplingWeight),
				new ActivityClusterArrayBalanceMetric(balanceWeight),
				new ActivityClusterArrayOverlapMetric(overlapWeight));
		return metric.compute(context, clusters, graph);
	}
}
