package org.processmining.activityclusterarraycreator.modifiers.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarraycreator.modifiers.ActivityClusterArrayModifier;
import org.processmining.activityclusterarraycreator.modifiers.ActivityClusterArrayModifierParameters;
import org.processmining.activityclusterarraycreator.utils.ClusterUtils;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.framework.plugin.PluginContext;

public class ActivityClusterArrayRandomModifier implements ActivityClusterArrayModifier {

	public final static String NAME = "Random";
	
	private ActivityClusterArray bestClusters;
	private Random random;
	private Map<Integer, List<Integer>> buckets;

	public ActivityClusterArray apply(PluginContext context, ActivityClusterArray clusters, CausalActivityGraph graph,
			ActivityClusterArrayModifierParameters parameters) {
		if (parameters.getNrOfClusters() >= clusters.getClusters().size() || parameters.getNrOfClusters() < 1) {
			return clusters;
		}

		bestClusters = null;
		random = new Random();
		buckets = new HashMap<Integer, List<Integer>>();

		List<List<Integer>> groupedClusters = new ArrayList<List<Integer>>();
		List<Integer> ungroupedClusters = new ArrayList<Integer>();
		for (int i = 0; i < clusters.getClusters().size(); i++) {
			ungroupedClusters.add(i);
		}

		findAllPossibleGroupings(context, groupedClusters, 0, ungroupedClusters, 0, ungroupedClusters.size(), clusters,
				graph, parameters);

		return bestClusters;
	}

	private void findAllPossibleGroupings(PluginContext context, List<List<Integer>> groupedClusters,
			int groupedClustersSize, List<Integer> ungroupedClusters, int ungroupedClustersIndex,
			int ungroupedClustersSize, ActivityClusterArray clusters, CausalActivityGraph graph,
			ActivityClusterArrayModifierParameters parameters) {

		if (groupedClustersSize + ungroupedClustersSize < parameters.getNrOfClusters()) {
			return;
		}

		if (ungroupedClustersSize > 0) {

			Integer cluster = ungroupedClusters.get(ungroupedClustersIndex);
			while (bestClusters == null) {
				int bucket = random.nextInt(parameters.getNrOfClusters());
				if (buckets.containsKey(bucket)) {
					buckets.get(bucket).add(cluster);
					findAllPossibleGroupings(context, groupedClusters, groupedClustersSize, ungroupedClusters,
							ungroupedClustersIndex + 1, ungroupedClustersSize - 1, clusters, graph, parameters);
					buckets.get(bucket).remove(cluster);
				} else if (groupedClustersSize < parameters.getNrOfClusters()) {
					List<Integer> groupedCluster = new ArrayList<Integer>();
					groupedCluster.add(cluster);
					groupedClusters.add(groupedCluster);
					buckets.put(bucket, groupedCluster);
					findAllPossibleGroupings(context, groupedClusters, groupedClustersSize + 1, ungroupedClusters,
							ungroupedClustersIndex + 1, ungroupedClustersSize - 1, clusters, graph, parameters);
					buckets.remove(bucket);
					groupedClusters.remove(groupedCluster);
				}
			}

		} else {

			ActivityClusterArray newClusters = clusters;

			for (List<Integer> group : groupedClusters) {
				Set<Set<XEventClass>> mergeClusters = new HashSet<Set<XEventClass>>();
				for (Integer index : group) {
					mergeClusters.add(clusters.getCluster(index));
				}
				newClusters = ClusterUtils.merge(newClusters, mergeClusters);
			}

			if (newClusters.isConsistent()) {
				bestClusters = newClusters;
			}

		}
	}
	
	public String getName() {
		return NAME;
	}
}
