package org.processmining.activityclusterarraycreator.plugins;

import java.util.Collection;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarraycreator.algorithms.ConvertCausalActivityGraphToActivityClusterArrayAlgorithm;
import org.processmining.activityclusterarraycreator.connections.ConvertCausalActivityGraphToActivityClusterArrayConnection;
import org.processmining.activityclusterarraycreator.dialogs.ConvertCausalActivityGraphToActivityClusterArrayDialog;
import org.processmining.activityclusterarraycreator.help.ConvertCausalActivityGraphToActivityClusterArrayHelp;
import org.processmining.activityclusterarraycreator.parameters.ConvertCausalActivityGraphToActivityClusterArrayParameters;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Create Clusters", parameterLabels = { "Causal Activity Graph", "Parameters" }, returnLabels = { "Activity Cluster Array" }, returnTypes = { ActivityClusterArray.class }, help = ConvertCausalActivityGraphToActivityClusterArrayHelp.TEXT)
public class ConvertCausalActivityGraphToActivityClusterArrayPlugin extends
		ConvertCausalActivityGraphToActivityClusterArrayAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Create Clusters, UI", requiredParameterLabels = { 0 })
	public ActivityClusterArray runUI(UIPluginContext context, CausalActivityGraph graph) {
		ConvertCausalActivityGraphToActivityClusterArrayParameters parameters = new ConvertCausalActivityGraphToActivityClusterArrayParameters(
				graph);
		ConvertCausalActivityGraphToActivityClusterArrayDialog dialog = new ConvertCausalActivityGraphToActivityClusterArrayDialog(
				context, graph, parameters);
		InteractionResult result = context.showWizard("Configure creation", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		return runConnection(context, graph, parameters);
	}

	/**
	 * @deprecated Use runUI() instead.
	 */
	@Deprecated
	public ActivityClusterArray convertUI(UIPluginContext context, CausalActivityGraph graph) {
		return runUI(context, graph);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Create Clusters, Default", requiredParameterLabels = { 0 })
	public ActivityClusterArray runDefault(PluginContext context, CausalActivityGraph graph) {
		ConvertCausalActivityGraphToActivityClusterArrayParameters parameters = new ConvertCausalActivityGraphToActivityClusterArrayParameters(
				graph);
		return runConnection(context, graph, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public ActivityClusterArray convertDefault(PluginContext context, CausalActivityGraph graph) {
		return runDefault(context, graph);
	}

	@PluginVariant(variantLabel = "Create Clusters, Parameters", requiredParameterLabels = { 0, 1 })
	public ActivityClusterArray run(PluginContext context, CausalActivityGraph graph,
			ConvertCausalActivityGraphToActivityClusterArrayParameters parameters) {
		return runConnection(context, graph, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public ActivityClusterArray convertParameters(PluginContext context, CausalActivityGraph graph,
			ConvertCausalActivityGraphToActivityClusterArrayParameters parameters) {
		return run(context, graph, parameters);
	}

	private ActivityClusterArray runConnection(PluginContext context, CausalActivityGraph graph,
			ConvertCausalActivityGraphToActivityClusterArrayParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<ConvertCausalActivityGraphToActivityClusterArrayConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						ConvertCausalActivityGraphToActivityClusterArrayConnection.class, context, graph);
				for (ConvertCausalActivityGraphToActivityClusterArrayConnection connection : connections) {
					if (connection.getObjectWithRole(ConvertCausalActivityGraphToActivityClusterArrayConnection.GRAPH)
							.equals(graph) && connection.getParameters().equals(parameters)) {
						return connection
								.getObjectWithRole(ConvertCausalActivityGraphToActivityClusterArrayConnection.CLUSTERS);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		ActivityClusterArray clusters = apply(context, graph, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(
					new ConvertCausalActivityGraphToActivityClusterArrayConnection(graph, clusters, parameters));
		}
		return clusters;
	}

}
